package com.wiccafe.garminreconnector

import android.Manifest
import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.app.Service
import android.bluetooth.*
import android.bluetooth.le.BluetoothLeScanner
import android.bluetooth.le.ScanCallback
import android.bluetooth.le.ScanFilter
import android.bluetooth.le.ScanResult
import android.bluetooth.le.ScanSettings
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Binder
import android.os.Build
import android.os.Handler
import android.os.IBinder
import android.os.Looper
import android.os.ParcelUuid
import androidx.core.app.ActivityCompat
import androidx.core.app.NotificationCompat
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale

// Enum to represent the specific connection status
enum class ConnectionStatus {
    DISCONNECTED,
    SCANNING,
    CONNECTING,
    CONNECTED,
    FAILED
}

class GarminReconnectService : Service() {

    private val binder = LocalBinder()

    private val _isServiceRunning = MutableStateFlow(false)
    val isServiceRunning = _isServiceRunning.asStateFlow()

    private val _connectionStatus = MutableStateFlow(ConnectionStatus.DISCONNECTED)
    val connectionStatus = _connectionStatus.asStateFlow()

    private val _logs = MutableStateFlow<List<String>>(emptyList())
    val logs = _logs.asStateFlow()

    private lateinit var bluetoothManager: BluetoothManager
    private var bluetoothAdapter: BluetoothAdapter? = null
    private var bluetoothLeScanner: BluetoothLeScanner? = null
    private var bluetoothGatt: BluetoothGatt? = null
    private val connectionHandler = Handler(Looper.getMainLooper())
    private val syncHandler = Handler(Looper.getMainLooper())
    private var retryCount = 0
    private var lastConnectedDeviceAddress: String? = null
    private var connectedDeviceName: String? = null

    inner class LocalBinder : Binder() {
        fun getService(): GarminReconnectService = this@GarminReconnectService
    }

    override fun onCreate() {
        super.onCreate()
        bluetoothManager = getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager
        bluetoothAdapter = bluetoothManager.adapter
        createNotificationChannel()
        log("Service created. Ready to scan for any Garmin device.")
    }

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        log("Service starting...")
        startForeground(NOTIFICATION_ID, createNotification("Service is running. Looking for Garmin device..."))
        _isServiceRunning.value = true
        startConnectionAttempt()
        syncHandler.post(syncRunnable)
        return START_STICKY
    }

    private val syncRunnable: Runnable = object : Runnable {
        override fun run() {
            if(_connectionStatus.value == ConnectionStatus.CONNECTED) {
                triggerGarminSync()
            }
            syncHandler.postDelayed(this, SYNC_INTERVAL_MS)
        }
    }

    private fun triggerGarminSync() {
        log("Attempting to trigger Garmin Connect sync...")
        val garminPackage = "com.garmin.android.apps.connectmobile"
        val launchIntent = packageManager.getLaunchIntentForPackage(garminPackage)
        if (launchIntent != null) {
            try {
                launchIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                startActivity(launchIntent)
                log("Sync successfully requested by launching Garmin Connect.")
            } catch (e: Exception) {
                log("Error launching Garmin Connect: ${e.message}")
            }
        } else {
            log("Garmin Connect app not found. Cannot trigger sync.")
        }
    }

    private fun startConnectionAttempt() {
        if (!checkPermissions("startConnectionAttempt")) {
            _connectionStatus.value = ConnectionStatus.FAILED
            stopSelf()
            return
        }

        lastConnectedDeviceAddress?.let { address ->
            log("Attempting to reconnect to known device: $address")
            val device = bluetoothAdapter?.getRemoteDevice(address)
            if (device != null) {
                connectToDevice(device)
                return
            }
        }

        val bondedGarmin = findBondedGarminDevice()
        if (bondedGarmin != null) {
            log("Found bonded Garmin device: ${bondedGarmin.name}. Attempting connection.")
            connectToDevice(bondedGarmin)
            return
        }

        log("No known or bonded device found. Starting a general scan.")
        startGeneralScan()
    }

    private fun findBondedGarminDevice(): BluetoothDevice? {
        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED) {
            return null
        }
        val bondedDevices = bluetoothAdapter?.bondedDevices
        return bondedDevices?.firstOrNull {
            it.name?.contains("Fenix", ignoreCase = true) == true ||
                    it.name?.contains("Forerunner", ignoreCase = true) == true ||
                    it.name?.contains("Venu", ignoreCase = true) == true ||
                    it.name?.contains("Instinct", ignoreCase = true) == true
        }
    }

    private fun startGeneralScan() {
        log("Permissions OK. Starting general scan for any Garmin device.")
        _connectionStatus.value = ConnectionStatus.SCANNING
        bluetoothLeScanner = bluetoothAdapter?.bluetoothLeScanner

        val garminServiceUuid = ParcelUuid.fromString(GARMIN_SERVICE_UUID)
        val scanFilter = ScanFilter.Builder().setServiceUuid(garminServiceUuid).build()
        val scanSettings = ScanSettings.Builder().setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY).build()

        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.BLUETOOTH_SCAN) == PackageManager.PERMISSION_GRANTED) {
            bluetoothLeScanner?.startScan(listOf(scanFilter), scanSettings, leScanCallback)
        }
    }

    private val leScanCallback = object : ScanCallback() {
        override fun onScanResult(callbackType: Int, result: ScanResult) {
            super.onScanResult(callbackType, result)
            if (!checkPermissions("onScanResult")) return

            val deviceName = result.device?.name ?: "Unknown Device"
            log("Garmin device found via scan: $deviceName")

            if (ActivityCompat.checkSelfPermission(this@GarminReconnectService, Manifest.permission.BLUETOOTH_SCAN) == PackageManager.PERMISSION_GRANTED) {
                bluetoothLeScanner?.stopScan(this)
            }

            connectToDevice(result.device)
        }
    }

    private fun connectToDevice(device: BluetoothDevice) {
        if (!checkPermissions("connectToDevice")) return
        log("Connecting to GATT server on ${device.name ?: "device"}...")
        _connectionStatus.value = ConnectionStatus.CONNECTING
        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.BLUETOOTH_CONNECT) == PackageManager.PERMISSION_GRANTED) {
            connectedDeviceName = device.name
            lastConnectedDeviceAddress = device.address
            bluetoothGatt = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                device.connectGatt(this, false, gattCallback, BluetoothDevice.TRANSPORT_LE)
            } else {
                device.connectGatt(this, false, gattCallback)
            }
        }
    }

    private val reconnectionRunnable = Runnable { handleDisconnection() }

    private val gattCallback = object : BluetoothGattCallback() {
        override fun onConnectionStateChange(gatt: BluetoothGatt, status: Int, newState: Int) {
            if (!checkPermissions("onConnectionStateChange")) return

            val deviceName = gatt.device?.name ?: "Garmin Device"
            if (newState == BluetoothProfile.STATE_CONNECTED) {
                connectionHandler.removeCallbacks(reconnectionRunnable)
                log("Successfully connected to $deviceName.")
                _connectionStatus.value = ConnectionStatus.CONNECTED
                updateNotification("Connected to $deviceName")
                retryCount = 0
            } else if (newState == BluetoothProfile.STATE_DISCONNECTED) {
                log("Transient disconnection from $deviceName. Status: $status. Starting grace period...")
                if (ActivityCompat.checkSelfPermission(this@GarminReconnectService, Manifest.permission.BLUETOOTH_CONNECT) == PackageManager.PERMISSION_GRANTED) {
                    gatt.close()
                }
                connectionHandler.postDelayed(reconnectionRunnable, GRACE_PERIOD_MS)
            }
        }
    }

    private fun handleDisconnection() {
        if (_connectionStatus.value == ConnectionStatus.CONNECTED) {
            // We were connected, but the grace period expired. Now we are officially disconnected.
            _connectionStatus.value = ConnectionStatus.DISCONNECTED
        }

        if (retryCount < MAX_RETRIES) {
            retryCount++
            log("Grace period over. Attempting to reconnect... (Attempt $retryCount/$MAX_RETRIES)")
            updateNotification("Disconnected. Reconnecting... (Attempt $retryCount)")
            startConnectionAttempt()
        } else {
            log("Max retries reached. Forcing general scan.")
            _connectionStatus.value = ConnectionStatus.FAILED
            lastConnectedDeviceAddress = null
            retryCount = 0
            connectionHandler.postDelayed({ startConnectionAttempt() }, RETRY_DELAY_MS)
        }
    }

    override fun onDestroy() {
        log("Service stopping...")
        connectionHandler.removeCallbacksAndMessages(null)
        syncHandler.removeCallbacksAndMessages(null)

        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.BLUETOOTH_CONNECT) == PackageManager.PERMISSION_GRANTED) {
            bluetoothGatt?.close()
        }
        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.BLUETOOTH_SCAN) == PackageManager.PERMISSION_GRANTED) {
            bluetoothLeScanner?.stopScan(leScanCallback)
        }
        bluetoothGatt = null

        // Broadcast the final "stopped" state
        _isServiceRunning.value = false
        _connectionStatus.value = ConnectionStatus.DISCONNECTED

        log("Service stopped and cleaned up.")
        super.onDestroy()
    }

    private fun log(message: String) {
        val timestamp = SimpleDateFormat("HH:mm:ss", Locale.getDefault()).format(Date())
        val newLogs = _logs.value.toMutableList()
        newLogs.add(0, "[$timestamp] $message")
        _logs.value = newLogs
    }

    private fun checkPermissions(caller: String): Boolean {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
            if (ActivityCompat.checkSelfPermission(this, Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED ||
                ActivityCompat.checkSelfPermission(this, Manifest.permission.BLUETOOTH_SCAN) != PackageManager.PERMISSION_GRANTED) {
                log("Permission check failed in '$caller': Bluetooth permissions not granted.")
                return false
            }
        }
        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            log("Permission check failed in '$caller': ACCESS_FINE_LOCATION not granted.")
            return false
        }
        return true
    }

    private fun createNotification(text: String): Notification {
        val notificationIntent = Intent(this, MainActivity::class.java)
        val pendingIntentFlag = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) PendingIntent.FLAG_IMMUTABLE else 0
        val pendingIntent = PendingIntent.getActivity(this, 0, notificationIntent, pendingIntentFlag)

        return NotificationCompat.Builder(this, CHANNEL_ID)
            .setContentTitle("Garmin Reconnector")
            .setContentText(text)
            .setSmallIcon(R.drawable.ic_launcher_foreground)
            .setContentIntent(pendingIntent)
            .setOngoing(true)
            .setPriority(NotificationCompat.PRIORITY_DEFAULT)
            .build()
    }

    private fun updateNotification(text: String) {
        val manager = getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
        manager.notify(NOTIFICATION_ID, createNotification(text))
    }

    private fun sendFailureNotification() {
        val manager = getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
        val deviceIdentifier = connectedDeviceName ?: "your Garmin device"
        val channelId = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val channel = NotificationChannel(FAILURE_CHANNEL_ID, "Failure Alerts", NotificationManager.IMPORTANCE_HIGH)
            manager.createNotificationChannel(channel)
            FAILURE_CHANNEL_ID
        } else { "" }

        val pendingIntentFlag = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) PendingIntent.FLAG_IMMUTABLE else 0
        val notificationIntent = Intent(this, MainActivity::class.java)
        val pendingIntent = PendingIntent.getActivity(this, 0, notificationIntent, pendingIntentFlag)

        val notification = NotificationCompat.Builder(this, channelId)
            .setContentTitle("Garmin Reconnection Failed")
            .setContentText("Could not reconnect to $deviceIdentifier after $MAX_RETRIES attempts.")
            .setSmallIcon(R.drawable.ic_launcher_foreground)
            .setContentIntent(pendingIntent)
            .setAutoCancel(true)
            .setPriority(NotificationCompat.PRIORITY_HIGH)
            .build()
        manager.notify(FAILURE_NOTIFICATION_ID, notification)
    }

    private fun createNotificationChannel() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val serviceChannel = NotificationChannel(
                CHANNEL_ID,
                "Garmin Reconnector Service Channel",
                NotificationManager.IMPORTANCE_DEFAULT
            )
            getSystemService(NotificationManager::class.java).createNotificationChannel(serviceChannel)
        }
    }

    override fun onBind(intent: Intent?): IBinder? {
        return binder
    }

    companion object {
        const val GARMIN_SERVICE_UUID = "00001530-1212-efde-1523-785feabcd123"
        const val MAX_RETRIES = 5
        const val RETRY_DELAY_MS = 5000L
        const val GRACE_PERIOD_MS = 30000L // 30 seconds grace period
        const val SYNC_INTERVAL_MS = 30 * 60 * 1000L // 30 minutes
        const val CHANNEL_ID = "GarminReconnectServiceChannel"
        const val FAILURE_CHANNEL_ID = "FailureChannel"
        const val NOTIFICATION_ID = 1
        const val FAILURE_NOTIFICATION_ID = 2
    }
}
