package com.wiccafe.garminreconnector

import android.Manifest
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.content.pm.PackageManager
import android.graphics.Color
import android.os.Build
import android.os.Bundle
import android.os.IBinder
import android.widget.Button
import android.widget.ScrollView
import android.widget.TextView
import androidx.activity.result.contract.ActivityResultContracts
import androidx.activity.viewModels
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.lifecycle.ViewModel
import androidx.lifecycle.lifecycleScope
import kotlinx.coroutines.Job
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch

// A ViewModel to hold UI state across configuration changes (like screen rotation)
class MainViewModel : ViewModel() {
    private val _logs = MutableStateFlow<List<String>>(emptyList())
    val logs = _logs.asStateFlow()

    fun updateLogs(newLogs: List<String>) {
        _logs.value = newLogs
    }
}

class MainActivity : AppCompatActivity() {

    private lateinit var logTextView: TextView
    private lateinit var logScrollView: ScrollView
    private lateinit var controlButton: Button
    private lateinit var statusTextView: TextView

    private var garminService: GarminReconnectService? = null
    private var isBound = false
    private var serviceCollectorJob: Job? = null

    // Get a reference to the ViewModel
    private val viewModel: MainViewModel by viewModels()

    private val connection = object : ServiceConnection {
        override fun onServiceConnected(className: ComponentName, service: IBinder) {
            val binder = service as GarminReconnectService.LocalBinder
            garminService = binder.getService()
            isBound = true
            observeServiceState()
        }

        // This callback is now primarily for unexpected crashes or system kills.
        override fun onServiceDisconnected(arg0: ComponentName) {
            isBound = false
            garminService = null
            updateButtonState(false)
            updateConnectionStatusUI(ConnectionStatus.DISCONNECTED)
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        logTextView = findViewById(R.id.logTextView)
        logScrollView = findViewById(R.id.logScrollView)
        controlButton = findViewById(R.id.controlButton)
        statusTextView = findViewById(R.id.statusTextView)

        controlButton.setOnClickListener { toggleService() }

        // Observe the ViewModel's state to restore logs on rotation.
        observeViewModel()
    }

    private fun observeViewModel() {
        lifecycleScope.launch {
            viewModel.logs.collectLatest { logs ->
                logTextView.text = logs.joinToString("\n")
                logScrollView.post { logScrollView.fullScroll(ScrollView.FOCUS_DOWN) }
            }
        }
    }

    override fun onStart() {
        super.onStart()
        // Always try to bind when the activity starts. This will also connect to a running service.
        Intent(this, GarminReconnectService::class.java).also { intent ->
            bindService(intent, connection, Context.BIND_AUTO_CREATE)
        }
    }

    override fun onResume() {
        super.onResume()
        if (!hasRequiredPermissions()) {
            requestPermissions()
        }
    }

    override fun onStop() {
        super.onStop()
        if (isBound) {
            unbindService(connection)
            isBound = false
        }
        serviceCollectorJob?.cancel()
    }

    private fun observeServiceState() {
        serviceCollectorJob?.cancel()
        serviceCollectorJob = lifecycleScope.launch {
            // This collector is now the single source of truth for the service running state.
            garminService?.isServiceRunning?.collect { isRunning ->
                updateButtonState(isRunning)
            }
        }
        lifecycleScope.launch {
            garminService?.logs?.collectLatest { logs ->
                // Push logs from the service into the ViewModel to persist them.
                viewModel.updateLogs(logs)
            }
        }
        lifecycleScope.launch {
            garminService?.connectionStatus?.collectLatest { status ->
                updateConnectionStatusUI(status)
            }
        }
    }

    private fun updateConnectionStatusUI(status: ConnectionStatus) {
        statusTextView.text = status.name
        when (status) {
            ConnectionStatus.DISCONNECTED -> statusTextView.setTextColor(Color.parseColor("#9CA3AF")) // Gray
            ConnectionStatus.SCANNING -> statusTextView.setTextColor(Color.parseColor("#60A5FA")) // Blue
            ConnectionStatus.CONNECTING -> statusTextView.setTextColor(Color.parseColor("#FBBF24")) // Amber
            ConnectionStatus.CONNECTED -> statusTextView.setTextColor(Color.parseColor("#34D399")) // Emerald
            ConnectionStatus.FAILED -> statusTextView.setTextColor(Color.parseColor("#F87171")) // Red
        }
    }

    private fun hasRequiredPermissions(): Boolean {
        val requiredPermissions = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
            listOf(Manifest.permission.BLUETOOTH_SCAN, Manifest.permission.BLUETOOTH_CONNECT, Manifest.permission.ACCESS_FINE_LOCATION)
        } else {
            listOf(Manifest.permission.ACCESS_FINE_LOCATION)
        }
        return requiredPermissions.all {
            ContextCompat.checkSelfPermission(this, it) == PackageManager.PERMISSION_GRANTED
        }
    }

    private fun toggleService() {
        if (!hasRequiredPermissions()) {
            requestPermissions()
            return
        }

        val serviceIntent = Intent(this, GarminReconnectService::class.java)
        val isRunning = garminService?.isServiceRunning?.value ?: false

        if (isRunning) {
            // THE FIX: Unbind first, then stop the service, then manually update the UI.
            // This prevents the "stuck button" race condition.
            if (isBound) {
                unbindService(connection)
                isBound = false
            }
            stopService(serviceIntent)
            updateButtonState(false)
            updateConnectionStatusUI(ConnectionStatus.DISCONNECTED)

        } else {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                startForegroundService(serviceIntent)
            } else {
                startService(serviceIntent)
            }
            // After starting, re-bind to establish the communication channel
            bindService(serviceIntent, connection, Context.BIND_AUTO_CREATE)
        }
    }

    private fun updateButtonState(running: Boolean) {
        controlButton.isEnabled = true
        controlButton.text = if (running) "Stop Monitoring Service" else "Start Monitoring Service"
        controlButton.setBackgroundColor(if (running) Color.parseColor("#FF6347") else Color.parseColor("#32CD32"))
    }

    private val requestPermissionLauncher =
        registerForActivityResult(ActivityResultContracts.RequestMultiplePermissions()) { permissions ->
            if (!hasRequiredPermissions()) {
                showInsufficientPermissionsDialog()
            }
        }

    private fun showInsufficientPermissionsDialog() {
        AlertDialog.Builder(this)
            .setTitle("Insufficient Permissions")
            .setMessage("This application requires Bluetooth and Location permissions to function. It will now close.")
            .setPositiveButton("Exit") { _, _ ->
                finish()
            }
            .setCancelable(false)
            .show()
    }

    private fun requestPermissions() {
        val permissionsToRequest = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
            arrayOf(
                Manifest.permission.BLUETOOTH_SCAN,
                Manifest.permission.BLUETOOTH_CONNECT,
                Manifest.permission.POST_NOTIFICATIONS,
                Manifest.permission.ACCESS_FINE_LOCATION
            )
        } else {
            arrayOf(
                Manifest.permission.BLUETOOTH,
                Manifest.permission.BLUETOOTH_ADMIN,
                Manifest.permission.ACCESS_FINE_LOCATION
            )
        }
        requestPermissionLauncher.launch(permissionsToRequest)
    }
}
